@@ -1,52 +1,53 @@
-#ifndef QUANTEDA // prevent redefining
-#define QUANTEDA
+#ifndef QUANTEDA4 // prevent redefining
+#define QUANTEDA4
+
+#ifndef ARMA_64BIT_WORD
+#define ARMA_64BIT_WORD
+#endif
 
 #include <RcppArmadillo.h>
-#include <RcppParallel.h>
 #include <unordered_map>
 #include <unordered_set>
 #include <limits>
 #include <algorithm>
-
-// [[Rcpp::plugins(cpp11)]]
-using namespace Rcpp;
-// [[Rcpp::depends(RcppParallel)]]
-using namespace RcppParallel;
-
-#define CLANG_VERSION (__clang_major__ * 10000 + __clang_minor__ * 100 + __clang_patchlevel__)
-
-// global setting for unordered_map and unordered_set 
-extern float GLOBAL_PATTERN_MAX_LOAD_FACTOR;
-extern float GLOBAL_NGRAMS_MAX_LOAD_FACTOR;
-
-// compiler has to be newer than clang 3.30 or gcc 4.8.1
-#if RCPP_PARALLEL_USE_TBB && (CLANG_VERSION >= 30300 || GCC_VERSION >= 40801) 
-#define QUANTEDA_USE_TBB true // tbb.h is loaded automatically by RcppParallel.h
+#include "tokens.h"
+#ifdef TBB
+#include <tbb/tbb.h>
+#ifdef ONETBB_SPEC_VERSION
+using namespace oneapi; // only Windows R 4.3.x or later
+#endif
+#define QUANTEDA_USE_TBB true
 #else
 #define QUANTEDA_USE_TBB false
 #endif
 
+// [[Rcpp::plugins(cpp11)]]
+using namespace Rcpp;
+
+// NOTE: used in textstats
+const float GLOBAL_PATTERN_MAX_LOAD_FACTOR = 0.05;
+const float GLOBAL_NGRAMS_MAX_LOAD_FACTOR = 0.25;
+
 namespace quanteda{
     
     typedef ListOf<IntegerVector> Tokens;
-    typedef std::vector<unsigned int> Text;
-    typedef std::vector<Text> Texts;
+    typedef XPtr<TokensObj> TokensPtr;
+
+    // typedef ListOf<IntegerVector> Tokens;
+    // typedef std::vector<unsigned int> Text;
+    // typedef std::vector<Text> Texts;
+    
+    typedef std::atomic<int> IntParam;
+    typedef std::atomic<unsigned int> UintParam;
+    typedef std::atomic<long> LongParam;
+    typedef std::atomic<double> DoubleParam;
     
 #if QUANTEDA_USE_TBB
-    typedef tbb::atomic<int> IntParam;
-    typedef tbb::atomic<unsigned int> UintParam;
-    typedef tbb::atomic<long> LongParam;
-    typedef tbb::atomic<double> DoubleParam;
     typedef tbb::concurrent_vector<int> IntParams;
     typedef tbb::concurrent_vector<long> LongParams;
     typedef tbb::concurrent_vector<double> DoubleParams;
     typedef tbb::concurrent_vector<std::string> StringParams;
-    typedef tbb::spin_mutex Mutex;
 #else
-    typedef int IntParam;
-    typedef unsigned int UintParam;
-    typedef long LongParam;
-    typedef double DoubleParam;
     typedef std::vector<int> IntParams;
     typedef std::vector<long> LongParams;
     typedef std::vector<double> DoubleParams;
@@ -57,8 +58,6 @@ namespace quanteda{
     
     typedef std::vector<unsigned int> Ngram;
     typedef std::vector<Ngram> Ngrams;
-    typedef std::string Type;
-    typedef std::vector<Type> Types;
     
     struct hash_ngram {
         std::size_t operator() (const Ngram &vec) const {
@@ -76,29 +75,27 @@ namespace quanteda{
         }
     };
     
+    typedef std::atomic<unsigned int> IdNgram;
+    typedef std::tuple<unsigned int, unsigned int, double> Triplet;
 #if QUANTEDA_USE_TBB
-    typedef tbb::atomic<unsigned int> IdNgram;
-    typedef tbb::concurrent_unordered_multimap<Ngram, UintParam, hash_ngram, equal_ngram> MultiMapNgrams;
-    typedef tbb::concurrent_unordered_map<Ngram, UintParam, hash_ngram, equal_ngram> MapNgrams;
+    typedef tbb::concurrent_unordered_multimap<Ngram, unsigned int, hash_ngram, equal_ngram> MultiMapNgrams;
+    typedef tbb::concurrent_unordered_map<Ngram, unsigned int, hash_ngram, equal_ngram> MapNgrams;
     typedef tbb::concurrent_unordered_set<Ngram, hash_ngram, equal_ngram> SetNgrams;
     typedef tbb::concurrent_vector<Ngram> VecNgrams;
     typedef tbb::concurrent_unordered_set<unsigned int> SetUnigrams;
-    typedef std::tuple<unsigned int, unsigned int, double> Triplet;
     typedef tbb::concurrent_vector<Triplet> Triplets; // for fcm_mt, ca_mt, wordfish_mt
 #else
-    typedef unsigned int IdNgram;
     typedef std::unordered_multimap<Ngram, unsigned int, hash_ngram, equal_ngram> MultiMapNgrams;
     typedef std::unordered_map<Ngram, unsigned int, hash_ngram, equal_ngram> MapNgrams;
     typedef std::unordered_set<Ngram, hash_ngram, equal_ngram> SetNgrams;
     typedef std::vector<Ngram> VecNgrams;
     typedef std::unordered_set<unsigned int> SetUnigrams;
-    typedef std::tuple<unsigned int, unsigned int, double> Triplet;
     typedef std::vector<Triplet> Triplets; // for fcm_mt, ca_mt, wordfish_mt
 #endif    
     
     
     inline String join_strings(CharacterVector &tokens_, 
-                       const String delim_ = " "){
+                               const String &delim_ = " "){
         
         if (tokens_.size() == 0) return "";
         String token_ = tokens_[0];
@@ -111,7 +108,7 @@ namespace quanteda{
     }
     
     inline std::string join_strings(std::vector<std::string> &tokens, 
-                            const std::string delim = " "){
+                                    const std::string &delim = " "){
         if (tokens.size() == 0) return "";
         std::string token = tokens[0];
         for (std::size_t i = 1; i < tokens.size(); i++) {
@@ -120,10 +117,11 @@ namespace quanteda{
         return token;
     }
     
-    inline String join_strings(std::vector<unsigned int> &tokens, 
-                       CharacterVector types_, 
-                       const String delim_ = " ") {
-        
+    // NOTE: used in textstats
+    inline String join_strings(std::vector<unsigned int> &tokens,
+                               const CharacterVector &types_,
+                               const String &delim_ = " ") {
+
         String token_("");
         if (tokens.size() > 0) {
             if (tokens[0] != 0) {
@@ -140,6 +138,52 @@ namespace quanteda{
         return token_;
     }
     
+    inline std::string join_strings(std::vector<unsigned int> &tokens,
+                                    const Types &types,
+                                    const std::string &delim = " ") {
+
+        std::string token("");
+        if (tokens.size() > 0) {
+            if (tokens[0] != 0) {
+                token += types[tokens[0] - 1];
+            }
+            for (std::size_t j = 1; j < tokens.size(); j++) {
+                if (tokens[j] != 0) {
+                    token += delim;
+                    token += types[tokens[j] - 1];
+                }
+            }
+        }
+        return token;
+    }
+    
+    inline bool is_encoded(String &delim_){
+        if (delim_.get_encoding() > 0) {
+            return true;
+        }
+        return false;
+    }
+    
+    inline bool is_encoded(CharacterVector &types_){
+        for (unsigned int i = 0; i < (unsigned int)types_.size(); i++) {
+            String type_ = types_[i];
+            if (type_.get_encoding() > 0) {
+                return true;
+            }
+        }
+        return false;
+    }
+    
+    inline CharacterVector encode(Types &types){
+        CharacterVector types_(types.size());
+        for (std::size_t i = 0; i < types.size(); i++) {
+            String type_ = types[i];
+            type_.set_encoding(CE_UTF8);
+            types_[i] = type_;
+        }
+        return(types_);
+    }
+    
     inline bool has_na(IntegerVector vec_) {
         for (unsigned int i = 0; i < (unsigned int)vec_.size(); ++i) {
             if (vec_[i] == NA_INTEGER) return true;
@@ -217,7 +261,7 @@ namespace quanteda{
         std::vector<unsigned int> ids = Rcpp::as< std::vector<unsigned int> >(ids_);
         std::vector<std::size_t> spans(patterns.size());
         for (size_t g = 0; g < std::min(patterns.size(), ids.size()); g++) {
-            map.insert(std::pair<Ngram, IdNgram>(patterns[g], ids[g]));
+            map.insert(std::pair<Ngram, unsigned int>(patterns[g], ids[g]));
             spans[g] = patterns[g].size();
         }
 
