@@ -40,6 +40,12 @@ test_that("print works", {
 test_that("corpus constructors works for kwic", {
     kw <- kwic(tokens(data_char_sampletext), "econom*")
 
+    # deprecated
+    lifecycle::expect_deprecated(
+        corpus(kw),
+        "`corpus.kwic()` was deprecated in quanteda 4.0.", fixed = TRUE
+    )
+    
     # split_context = TRUE, extract_keyword = TRUE
     corp <- corpus(kw, split_context = TRUE, extract_keyword = TRUE)
     expect_is(corp, "corpus")
@@ -64,20 +70,30 @@ test_that("corpus constructors works for kwic", {
     )
 
     # test text handling for punctuation - there should be no space before the ?
-    corp <- tokens(data_char_sampletext, what = "word", remove_separators = FALSE) %>%
-      kwic("econom*", window = 10, separator = "") %>%
-      corpus(split_context = FALSE, extract_keyword = FALSE)
+    corp1 <- tokens(data_char_sampletext, what = "word", remove_separators = FALSE) |>
+      kwic("econom*", window = 10, separator = "") |>
+      corpus(split_context = FALSE, extract_keyword = FALSE, concatenator = "")
     expect_identical(
-        as.character(corp)[2],
+        as.character(corp1)[2],
         c("text1.L390" = "it is decimating the domestic economy? As we are tired ")
     )
+    
+    # concatenator is working
+    corp2 <- tokens(data_char_sampletext, what = "word", remove_separators = TRUE) |>
+        kwic("econom*", window = 5) |>
+        corpus(split_context = FALSE, extract_keyword = FALSE)
+    expect_identical(
+        as.character(corp2)[2],
+        c("text1.L202" = "it is decimating the domestic economy ? As we are tired")
+    )
 
     # ; and !
     txt1 <- c("This is; a test!")
     toks1 <- tokens(txt1, what = "word", remove_separators = FALSE)
     kw1 <- kwic(toks1, "a", window = 10, separator = "")
     expect_equivalent(
-        as.character(corpus(kw1, split_context = FALSE)), txt1
+        as.character(corpus(kw1, split_context = FALSE, concatenator = "")), 
+        txt1
     )
 
     # quotes
@@ -85,25 +101,29 @@ test_that("corpus constructors works for kwic", {
     toks2 <- tokens(txt2, what = "word", remove_separators = FALSE)
     kw2 <- kwic(toks2, "a", window = 10, separator = "")
     expect_equivalent(
-        as.character(corpus(kw2, split_context = FALSE)), txt2
+        as.character(corpus(kw2, split_context = FALSE, concatenator = "")), 
+        txt2
     )
     txt3 <- "This \"is\" only a test!"
     toks3 <- tokens(txt3, what = "word", remove_separators = FALSE)
     kw3 <- kwic(toks3, "a", window = 10, separator = "")
     expect_equivalent(
-        as.character(corpus(kw3, split_context = FALSE)), txt3
+        as.character(corpus(kw3, split_context = FALSE, concatenator = "")), 
+        txt3
     )
     txt4 <- 'This "is" only (a) test!'
     toks4 <- tokens(txt4, what = "word", remove_separators = FALSE)
     kw4 <- kwic(toks4, "a", window = 10, separator = "")
     expect_equivalent(
-        as.character(corpus(kw4, split_context = FALSE)), txt4
+        as.character(corpus(kw4, split_context = FALSE, concatenator = "")), 
+        txt4
     )
     txt5 <- "This is only (a) test!"
     toks5 <- tokens(txt5, what = "word", remove_separators = FALSE)
     kw5 <- kwic(toks5, "a", window = 10, separator = "")
     expect_equivalent(
-        as.character(corpus(kw5, split_context = FALSE)), txt5
+        as.character(corpus(kw5, split_context = FALSE, concatenator = "")), 
+        txt5
     )
     expect_error(corpus(kw, split_context = logical(), extract_keyword = FALSE))
     expect_error(corpus(kw, extract_keyword = logical()))
@@ -264,8 +284,8 @@ test_that("corpus works on dplyr grouped data.frames (#1232)", {
                    some_ints = 1L:6L,
                    text = paste0("This is text number ", 1:6, "."),
                    stringsAsFactors = FALSE,
-                   row.names = paste0("fromDf_", 1:6)) %>%
-        dplyr::group_by(letter_factor) %>%
+                   row.names = paste0("fromDf_", 1:6)) |>
+        dplyr::group_by(letter_factor) |>
         dplyr::mutate(n_group = dplyr::n())
     expect_output(
         print(corpus(df_grouped)),
@@ -401,28 +421,28 @@ test_that("correctly handle data.frame with improper column names (#1388)", {
     # when one column name is NA
     names(df)[3] <- NA
     expect_equal(
-        corpus(df) %>% docvars() %>% names(),
+        corpus(df) |> docvars() |> names(),
         c("dvar1", "V2", "dvar3")
     )
 
     # when two column names are NA
     names(df)[3:4] <- NA
     expect_equal(
-        corpus(df) %>% docvars() %>% names(),
+        corpus(df) |> docvars() |> names(),
         c("dvar1", "V2", "V3")
     )
 
     # when one column name is blank
     names(df)[3:4] <- c("dv", "")
     expect_equal(
-        corpus(df) %>% docvars() %>% names(),
+        corpus(df) |> docvars() |> names(),
         c("dvar1", "dv", "V3")
     )
 
     # when two column names are blank
     names(df)[3:4] <- ""
     expect_equal(
-        corpus(df) %>% docvars() %>% names(),
+        corpus(df) |> docvars() |> names(),
         c("dvar1", "V2", "V3")
     )
 })
