@@ -187,7 +187,7 @@ test_that("as.fcm is working", {
     expect_true(is.fcm(as.fcm(as(as(mt1, "CsparseMatrix"), "triangularMatrix"))))
     expect_true(is.fcm(as.fcm(as(mt1, "dgCMatrix"))))
     expect_true(is.fcm(as.fcm(as(mt1, "TsparseMatrix"))))
-    expect_true(is.fcm(as.fcm(Matrix(mt1, sparse = FALSE))))
+    expect_true(is.fcm(as.fcm(Matrix::Matrix(mt1, sparse = FALSE))))
 
     mt2 <- matrix(c(1, 3, 1, 2, 2,
                     0, 1, 2, 0, 1,
@@ -199,6 +199,19 @@ test_that("as.fcm is working", {
 
     expect_error(as.fcm(mt2),
                 "matrix must have the same rownames and colnames")
-    expect_error(as.fcm(Matrix(mt2, sparse = FALSE)),
+    expect_error(as.fcm(Matrix::Matrix(mt2, sparse = FALSE)),
                 "matrix must have the same rownames and colnames")
 })
+
+test_that("Compatible with Matrix 1.5-5 changes in dimnames", {
+    dfmat <- dfm(tokens(c("a aa a", "a aaa aa aa")))
+    fcmat <- fcm(dfmat)
+    expect_equal(
+        featnames(dfm_remove(dfmat, "a*")),
+        character(0)
+    )
+    expect_equal(
+        featnames(fcm_remove(fcmat, "a*")),
+        character(0)
+    )
+})
