@@ -1,13 +1,5 @@
 quanteda_options("tokens_tokenizer_word" = "word3")
 
-test_that("as.tokens list version works as expected", {
-    txt <- c(doc1 = "The first sentence is longer than the second.",
-             doc2 = "Told you so.")
-    toks_list <- as.list(tokens(txt))
-    toks <- tokens(txt)
-    expect_equivalent(as.tokens(toks_list), toks)
-})
-
 test_that("tokens indexing works as expected", {
     toks <- tokens(c(d1 = "one two three",
                      d2 = "four five six",
@@ -31,13 +23,13 @@ test_that("tokens indexing works as expected", {
 })
 
 test_that("tokens_recompile combine duplicates is working", {
-    toksh <- tokens(c(one = "a b c d A B C D", two = "A B C d"))
-    expect_equivalent(attr(toksh, "types"),
+    toks <- tokens(c(one = "a b c d A B C D", two = "A B C d"))
+    expect_equivalent(attr(toks, "types"),
                       c("a", "b", "c", "d", "A", "B", "C", "D"))
-    expect_equivalent(attr(tokens_tolower(toksh), "types"),
+    expect_equivalent(attr(tokens_tolower(toks), "types"),
                       c("a", "b", "c", "d"))
-    attr(toksh, "types") <- char_tolower(attr(toksh, "types"))
-    expect_equivalent(attr(quanteda:::tokens_recompile(toksh), "types"),
+    attr(toks, "types") <- char_tolower(attr(toks, "types"))
+    expect_equivalent(attr(quanteda:::tokens_recompile(toks), "types"),
                       c("a", "b", "c", "d"))
 
 })
@@ -216,8 +208,13 @@ test_that("c() works with tokens", {
                       d2 = "Here is the second sample document."))
     toks2 <- tokens(c(d3 = "And the third document."))
     toks3 <- tokens(c(d4 = "This is sample document 4."))
-    toks4 <- tokens(c(d1 = "This is sample document five!"))
-
+    toks4 <- tokens(c(d5 = "This is sample document five!"))
+    
+    expect_error(
+        c(toks1, list()),
+        "Cannot combine different types of objects"
+    )
+    
     expect_equal(
         c(toks1),
         toks1
@@ -229,12 +226,12 @@ test_that("c() works with tokens", {
     )
 
     expect_equal(
-        c(toks1, toks2, toks3),
-        toks1 + toks2 + toks3
+        c(toks1, toks2, toks3, toks4),
+        toks1 + toks2 + toks3 + toks4
     )
 
     expect_error(
-        c(toks1, toks4),
+        c(toks1, toks1),
         "Cannot combine tokens with duplicated document names"
     )
 
@@ -244,8 +241,8 @@ test_that("c() works with tokens", {
               tokens(data_corpus_inaugural[6:10]))
 
     expect_equivalent(
-         toks,
-         tokens(data_corpus_inaugural[1:10])
+         as.list(toks),
+         as.list(tokens(data_corpus_inaugural[1:10]))
     )
 
     expect_equal(
@@ -416,23 +413,23 @@ test_that("test that features remove by tokens.tokens is comparable to tokens.ch
     toks4 <- as.tokens(stringi::stri_split_fixed(chars[4], " "))
     toks5 <- as.tokens(stringi::stri_split_fixed(chars[5], " "))
 
-    expect_equal(tokens(chars[1], remove_numbers = TRUE) %>% as.list(),
-                 tokens(toks1, remove_numbers = TRUE) %>% as.list())
+    expect_equal(tokens(chars[1], remove_numbers = TRUE) |> as.list(),
+                 tokens(toks1, remove_numbers = TRUE) |> as.list())
 
-    expect_equal(tokens(chars[1], remove_punct = TRUE) %>% as.list(),
-                 tokens(toks1, remove_punct = TRUE) %>% as.list())
+    expect_equal(tokens(chars[1], remove_punct = TRUE) |> as.list(),
+                 tokens(toks1, remove_punct = TRUE) |> as.list())
 
-    expect_equal(tokens(chars[1], remove_separator = TRUE) %>% as.list(),
-                 tokens(toks1, remove_separator = TRUE) %>% as.list())
+    expect_equal(tokens(chars[1], remove_separator = TRUE) |> as.list(),
+                 tokens(toks1, remove_separator = TRUE) |> as.list())
 
-    expect_equal(tokens(chars[1], remove_symbols = TRUE) %>% as.list(),
-                 tokens(toks1, remove_symbols = TRUE) %>% as.list())
+    expect_equal(tokens(chars[1], remove_symbols = TRUE) |> as.list(),
+                 tokens(toks1, remove_symbols = TRUE) |> as.list())
 
-    expect_equal(tokens(chars[4], remove_url = TRUE) %>% as.list(),
-                 tokens(toks4, remove_url = TRUE) %>% as.list())
+    expect_equal(tokens(chars[4], remove_url = TRUE) |> as.list(),
+                 tokens(toks4, remove_url = TRUE) |> as.list())
 
-    expect_equal(tokens(chars[3], split_hyphens = TRUE) %>% as.list(),
-                 tokens(toks3, split_hyphens = TRUE) %>% as.list())
+    expect_equal(tokens(chars[3], split_hyphens = TRUE) |> as.list(),
+                 tokens(toks3, split_hyphens = TRUE) |> as.list())
 
     # This fails because there is not separator in toks
     # expect_equal(tokens(chars[1], remove_symbols = TRUE, remove_separator = FALSE),
@@ -440,20 +437,6 @@ test_that("test that features remove by tokens.tokens is comparable to tokens.ch
 
 })
 
-test_that("split_hyphens is working correctly", {
-    corp <- data_corpus_inaugural[1:2]
-    toks <- tokens(corp)
-
-    suppressWarnings({
-    expect_equal(dfm(corp), dfm(toks))
-    expect_equal(dfm(corp, remove_punct = TRUE), dfm(toks, remove_punct = TRUE))
-    expect_equal(
-        setdiff(featnames(dfm(corp)), featnames(dfm(toks))),
-        character()
-    )
-    })
-})
-
 test_that("tokens works as expected with NA, and blanks", {
     expect_equal(
         as.list(tokens(c("one", "two", ""))),
@@ -536,6 +519,7 @@ test_that("combined tokens objects have all the attributes", {
     expect_identical(names(attributes(c(toks1, toks4))),
                      names(attributes(toks1)))
     expect_identical(attr(c(toks1, toks4), "meta")$object$what, "word")
+    expect_identical(attr(c(toks1, toks4), "meta")$object$tokenizer, "tokenize_word3")
     expect_identical(attr(c(toks1, toks4), "meta")$object$concatenator, "_")
     expect_identical(attr(c(toks1, toks4), "meta")$object$ngram, c(1L))
     expect_identical(attr(c(toks1, toks4), "meta")$object$skip, c(0L))
@@ -544,6 +528,7 @@ test_that("combined tokens objects have all the attributes", {
     expect_identical(names(attributes(c(toks1, toks5))),
                      names(attributes(toks1)))
     expect_identical(attr(c(toks1, toks5), "meta")$object$what, "word")
+    expect_identical(attr(c(toks1, toks5), "meta")$object$tokenizer, "tokenize_word3")
     expect_identical(attr(c(toks1, toks5), "meta")$object$concatenator, "_")
     expect_identical(attr(c(toks1, toks5), "meta")$object$ngram, 1L)
     expect_identical(attr(c(toks1, toks5), "meta")$object$skip, 0L)
@@ -585,15 +570,15 @@ test_that("tokens.tokens(x, split_hyphens = TRUE) behaves same as tokens.charact
     # issue #1498
     txt <- "Auto-immune system."
     expect_identical(
-        as.character(tokens(txt, split_hyphens = FALSE) %>% tokens(split_hyphens = TRUE)),
+        as.character(tokens(txt, split_hyphens = FALSE) |> tokens(split_hyphens = TRUE)),
         c("Auto", "-", "immune", "system", ".")
     )
 
     txt <- c("There's shrimp-kabobs, shrimp creole. Deep-deep-fried, stir-fried.",
              "Stir-fried shrimp.")
     expect_identical(
-        tokens(txt, split_hyphens = TRUE) %>% as.list(),
-        tokens(txt, split_hyphens = FALSE) %>% tokens(split_hyphens = TRUE) %>% as.list()
+        tokens(txt, split_hyphens = TRUE) |> as.list(),
+        tokens(txt, split_hyphens = FALSE) |> tokens(split_hyphens = TRUE) |> as.list()
     )
 })
 
@@ -616,14 +601,19 @@ test_that("types are encoded when necessarly", {
 test_that("tokens verbose = TRUE produces expected messages", {
     expect_message(
         tokens(c("one two three", "four five."), verbose = TRUE),
-        "starting tokenization"
+        "^Creating a tokens from a character object"
+    )
+    expect_message(
+        tokens(corpus(c("one two three", "four five.")), xptr = TRUE, verbose = TRUE),
+        "^Creating a tokens_xptr from a corpus object"
     )
 })
 
 test_that("types<- with wrong value generates error", {
     toks <- tokens(c("one two three", "four five."))
     expect_error(
-        quanteda:::`types<-.tokens`(toks, value = 1:6),
+        # quanteda:::`types<-.tokens`(toks, value = 1:6),
+        quanteda:::`types<-`(toks, value = 1:6),
         "replacement value must be character"
     )
 })
@@ -716,6 +706,7 @@ test_that("tokens.tokens(x, remove_symbols = TRUE, verbose = TRUE) works as expe
 })
 
 test_that("tokens.tokens(x, remove_separators = TRUE, verbose = TRUE) works as expected (#1683)", {
+    skip("the verbose message has been changed")
     expect_message(
         tokens(tokens("Removing separators", remove_separators = FALSE, what = "word1"),
                remove_separators = TRUE, verbose = TRUE),
@@ -774,38 +765,38 @@ test_that("test that what = \"word\" works the same as \"word2\"", {
     chars <- c("a b c 12345 ! @ # $ % ^ & * ( ) _ + { } | : \' \" < > ? ! , . \t \n \u2028 \u00A0 \u2003",
                "#tag @user", "abc be-fg hi 100kg 2017", "a b c d e")
 
-    expect_equal(tokens(chars, what = "word", remove_numbers = TRUE) %>% as.list(),
-                 tokens(chars, what = "word1", remove_numbers = TRUE) %>% as.list())
-    expect_equal(tokens(chars, what = "word", remove_numbers = TRUE) %>% as.list(),
-                 tokens(chars, what = "word1", remove_numbers = TRUE) %>% as.list())
+    expect_equal(tokens(chars, what = "word", remove_numbers = TRUE) |> as.list(),
+                 tokens(chars, what = "word1", remove_numbers = TRUE) |> as.list())
+    expect_equal(tokens(chars, what = "word", remove_numbers = TRUE) |> as.list(),
+                 tokens(chars, what = "word1", remove_numbers = TRUE) |> as.list())
 
-    expect_equal(tokens(chars, what = "word", remove_symbols = TRUE) %>% as.list(),
-                 tokens(chars, what = "word1", remove_symbols = TRUE) %>% as.list())
-    expect_equal(tokens(chars, what = "word", remove_symbols = TRUE) %>% as.list(),
-                 tokens(chars, what = "word1", remove_symbols = TRUE) %>% as.list())
+    expect_equal(tokens(chars, what = "word", remove_symbols = TRUE) |> as.list(),
+                 tokens(chars, what = "word1", remove_symbols = TRUE) |> as.list())
+    expect_equal(tokens(chars, what = "word", remove_symbols = TRUE) |> as.list(),
+                 tokens(chars, what = "word1", remove_symbols = TRUE) |> as.list())
 
-    expect_equal(tokens(chars, what = "word", remove_punct = TRUE) %>% as.list(),
-                 tokens(chars, what = "word1", remove_punct = TRUE) %>% as.list())
-    expect_equal(tokens(chars, what = "word", remove_punct = TRUE) %>% as.list(),
-                 tokens(chars, what = "word1", remove_punct = TRUE) %>% as.list())
+    expect_equal(tokens(chars, what = "word", remove_punct = TRUE) |> as.list(),
+                 tokens(chars, what = "word1", remove_punct = TRUE) |> as.list())
+    expect_equal(tokens(chars, what = "word", remove_punct = TRUE) |> as.list(),
+                 tokens(chars, what = "word1", remove_punct = TRUE) |> as.list())
 
-    expect_equal(tokens(chars, what = "word", remove_punct = TRUE, split_tags = TRUE) %>% as.list(),
-                 tokens(chars, what = "word1", remove_punct = TRUE, split_tags = TRUE) %>% as.list())
-    expect_equal(tokens(chars, what = "word", remove_punct = TRUE, split_tags = TRUE) %>% as.list(),
-                 tokens(chars, what = "word1", remove_punct = TRUE, split_tags = TRUE) %>% as.list())
+    expect_equal(tokens(chars, what = "word", remove_punct = TRUE, split_tags = TRUE) |> as.list(),
+                 tokens(chars, what = "word1", remove_punct = TRUE, split_tags = TRUE) |> as.list())
+    expect_equal(tokens(chars, what = "word", remove_punct = TRUE, split_tags = TRUE) |> as.list(),
+                 tokens(chars, what = "word1", remove_punct = TRUE, split_tags = TRUE) |> as.list())
     suppressWarnings(
-        expect_equal(tokens(chars, what = "word", remove_punct = FALSE, split_tags = TRUE) %>% as.list(),
-                     tokens(chars, what = "word1", remove_punct = FALSE, split_tags = TRUE) %>% as.list())
+        expect_equal(tokens(chars, what = "word", remove_punct = FALSE, split_tags = TRUE) |> as.list(),
+                     tokens(chars, what = "word1", remove_punct = FALSE, split_tags = TRUE) |> as.list())
     )
     suppressWarnings(
-        expect_equal(tokens(chars, what = "word", remove_punct = FALSE, split_tags = TRUE) %>% as.list(),
-                     tokens(chars, what = "word1", remove_punct = FALSE, split_tags = TRUE) %>% as.list())
+        expect_equal(tokens(chars, what = "word", remove_punct = FALSE, split_tags = TRUE) |> as.list(),
+                     tokens(chars, what = "word1", remove_punct = FALSE, split_tags = TRUE) |> as.list())
     )
 
-    expect_equal(tokens(chars, what = "word", split_hyphens = TRUE) %>% as.list(),
-                 tokens(chars, what = "word1", split_hyphens = TRUE) %>% as.list())
-    expect_equal(tokens(chars, what = "word", split_hyphens = TRUE) %>% as.list(),
-                 tokens(chars, what = "word1", split_hyphens = TRUE) %>% as.list())
+    expect_equal(tokens(chars, what = "word", split_hyphens = TRUE) |> as.list(),
+                 tokens(chars, what = "word1", split_hyphens = TRUE) |> as.list())
+    expect_equal(tokens(chars, what = "word", split_hyphens = TRUE) |> as.list(),
+                 tokens(chars, what = "word1", split_hyphens = TRUE) |> as.list())
 })
 
 
@@ -985,14 +976,12 @@ test_that("special1 functions are working", {
                                      split_hyphens = FALSE, split_tags = FALSE),
         "_ht_quanteda _ht_q_hy_x _ht_q_y _ht_q100 _ht_q"
     )
-    toks1 <- list(1:5)
-    attr(toks1, "types") <- c("_ht_quanteda", "_ht_q-x", "_ht_q_y", "_ht_q100", "_ht_q")
+    toks1 <- as.tokens(list(c("_ht_quanteda", "_ht_q-x", "_ht_q_y", "_ht_q100", "_ht_q")))
     expect_identical(
         attr(quanteda:::restore_special1(toks1, split_hyphens = TRUE, split_tags = FALSE), "types"),
         c("#quanteda", "#q-x", "#q_y", "#q100", "#q")
     )
-    toks2 <- list(1:5)
-    attr(toks2, "types") <- c("_ht_quanteda", "_ht_q_hy_x", "_ht_q_y", "_ht_q100", "_ht_q")
+    toks2 <- as.tokens(list(c("_ht_quanteda", "_ht_q_hy_x", "_ht_q_y", "_ht_q100", "_ht_q")))
     expect_identical(
         attr(quanteda:::restore_special1(toks2, split_hyphens = FALSE, split_tags = FALSE), "types"),
         c("#quanteda", "#q-x", "#q_y", "#q100", "#q")
@@ -1027,10 +1016,10 @@ test_that("tokenizing Japanese with URLs works", {
 })
 
 test_that("Non-ASCII hashtags are preserved", {
-    txt <- c(d1 = "オリンピック延期決定！ #政治 #安部政権")
+    txt <- c(d1 = "オリンピック延期決定！ #政治 #安倍政権")
     expect_identical(
         as.list(tokens(txt, what = "word")),
-        list(d1 = c("オリンピック", "延期", "決定", "！", "#政治", "#安部政権"))
+        list(d1 = c("オリンピック", "延期", "決定", "！", "#政治", "#安倍政権"))
     )
 })
 
@@ -1046,7 +1035,7 @@ test_that("emails address is preserved", {
     # prevents test failing on Ubuntu 20.04 on GitHub Actions
     skip_if(
         as.numeric(stringi::stri_info()$Unicode.version) > 10 &&
-            as.numeric(stringi::stri_info()$ICU.version) > 61.1
+        as.numeric(stringi::stri_info()$ICU.version) > 61.1
     )
     txt <- c(d1 = "support@quanteda.io K.Watanabe@qi1234.co.jp")
     expect_identical(
@@ -1091,20 +1080,21 @@ test_that("old preserve_special works", {
 })
 
 test_that("output is correct for word1", {
+    
     expect_message(
-        tmp <- tokens(data_char_ukimmig2010, what = "word1", split_hyphens = FALSE, verbose = TRUE),
+        toks <- tokens(data_char_ukimmig2010, what = "word1", split_hyphens = FALSE, verbose = TRUE),
         "preserving hyphens"
     )
     expect_message(
-        tmp <- tokens(data_char_ukimmig2010, what = "word1", split_hyphens = FALSE, verbose = TRUE),
+        toks <- tokens(data_char_ukimmig2010, what = "word1", split_hyphens = FALSE, verbose = TRUE),
         "Finished constructing tokens from 9 documents"
     )
     expect_message(
-        tmp <- tokens(data_char_ukimmig2010, what = "word1", split_hyphens = FALSE, verbose = TRUE),
-        "^Creating a tokens object from a character input"
+        toks <- tokens(data_char_ukimmig2010, what = "word1", split_hyphens = FALSE, verbose = TRUE),
+        "^Creating a tokens from a character object"
     )
     expect_message(
-        tmp <- tokens(data_char_ukimmig2010, what = "sentence", verbose = TRUE),
+        toks <- tokens(data_char_ukimmig2010, what = "sentence", verbose = TRUE),
         "segmenting into sentences"
     )
 })
@@ -1134,4 +1124,102 @@ test_that("edge case usernames are correctly recognized", {
     )
 })
 
+test_that("cancatenator is working", {
+    
+    txt <- c(d1 = "The United States is bordered by the Atlantic Ocean and the Pacific Ocean.",
+             d2 = "The Supreme Court of the United States is seldom in a united state.")
+    
+    # construct
+    toks <- tokens(txt, remove_punct = TRUE, concatenator = " ")
+    expect_error(
+        tokens(txt, remove_punct = TRUE, concatenator = c(" ", " ")),
+        "The length of concatenator must be 1"
+    )
+    expect_equal(
+        concatenator(toks),
+        " "
+    )
+    expect_equal(
+        concat(toks),
+        " "
+    )
+    
+    # compound
+    dict <- dictionary(list(Countries = c("* States", "Federal Republic of *"),
+                            Oceans = c("* Ocean")), tolower = FALSE)
+    toks <- tokens_compound(toks, dict, concatenator = "_")
+    expect_equal(
+        concatenator(toks),
+        "_"
+    )
+    expect_equal(
+        concat(toks),
+        "_"
+    )
+    expect_equal(
+        ntoken(tokens_select(toks, c("United_States"))),
+        c(d1 = 1, d2 = 1)
+    )
+    
+    # update
+    toks <- tokens(toks, concatenator = "+")
+    expect_error(
+        tokens(toks, concatenator = c(" ", " ")),
+        "The length of concatenator must be 1"
+    )
+    expect_equal(
+        concatenator(toks),
+        "+"
+    )
+    expect_equal(
+        concat(toks),
+        "+"
+    )
+    expect_equal(
+        ntoken(tokens_select(toks, "United+States")),
+        c(d1 = 1, d2 = 1)
+    )
+})
+
+test_that("cancatenator is passed to the downstream", {
+    
+    txt <- c(d1 = "The United States is bordered by the Atlantic Ocean and the Pacific Ocean.",
+             d2 = "The Supreme Court of the United States is seldom in a united state.")
+    
+    dict <- dictionary(list(Countries = c("* States", "Federal Republic of *"),
+                            Oceans = c("* Ocean")), tolower = FALSE)
+    
+    # construct
+    
+    toks <- tokens(txt, remove_punct = TRUE, concatenator = "+")
+    
+    toks_dict <- tokens_lookup(toks, dict, append_key = TRUE)
+    expect_true("United+States/Countries" %in% types(toks_dict))
+    expect_identical(concat(toks_dict), "+")
+    
+    toks_ngram <- tokens_ngrams(toks)
+    expect_true("United+States" %in% types(toks_ngram))
+    expect_identical(concat(toks_ngram), "+")
+    
+    toks_comp <- tokens_compound(toks, dict)
+    expect_true("United+States" %in% types(toks_comp))
+    expect_identical(concat(toks_comp), "+")
+    
+    # re-construct with different concatantor
+    
+    toks2 <- tokens(toks, remove_punct = TRUE, concatenator = "*")
+    
+    toks_dict2 <- tokens_lookup(toks2, dict, append_key = TRUE)
+    expect_true("United*States/Countries" %in% types(toks_dict2))
+    expect_identical(concat(toks_dict2), "*")
+    
+    toks_ngram2 <- tokens_ngrams(toks2)
+    expect_true("United*States" %in% types(toks_ngram2))
+    expect_identical(concat(toks_ngram2), "*")
+    
+    toks_comp2 <- tokens_compound(toks2, dict)
+    expect_true("United*States" %in% types(toks_comp2))
+    expect_identical(concat(toks_comp2), "*")
+    
+})
 quanteda_options(reset = TRUE)
