@@ -15,61 +15,45 @@ test_that("test that ngrams produces the results from Guthrie 2006", {
           'killed_in_fighting', 'killed_ongoing_fighting', 
           'in_ongoing_fighting')
       
-      expect_equivalent(setdiff(
+      expect_setequal(
           as.list(tokens_ngrams(toks, n=2, skip=0))[[1]],
           bi_grams
-          ), character(0)
       )
       
-      expect_equivalent(setdiff(
+      expect_setequal(
           as.list(tokens_ngrams(toks, n=2, skip=0:2))[[1]],
           two_skip_bi_grams
-          ), character(0)
       )
       
-      expect_equivalent(setdiff(
+      expect_setequal(
           as.list(tokens_ngrams(toks, n=3, skip=0))[[1]],
           tri_grams
-          ), character(0)
       )
       
-      expect_equivalent(setdiff(
+      expect_setequal(
           as.list(tokens_ngrams(toks, n=3, skip=0:2))[[1]],
           two_skip_tri_grams
-          ), character(0)
       )
       
-      expect_equivalent(setdiff(
+      expect_setequal(
         as.list(tokens_ngrams(toks, n = 2:3))[[1]],
         c(bi_grams, tri_grams)
-        ), character(0)
       )
 
-      expect_equivalent(setdiff(
+      expect_setequal(
           as.list(suppressWarnings(tokens_ngrams(toks, n = 2:3)))[[1]],
           c(bi_grams, tri_grams)
-      ), character(0)
       )
 })
 
-test_that("test `tokens_ngrams` on characters", {
-    ngms <- quanteda:::tokens_ngrams_character(c('insurgents', 'killed', 'in', 'ongoing', 'fighting'))
-    charNgms <- char_ngrams(c('insurgents','killed', 'in', 'ongoing', 'fighting'))
+test_that("char_ngrams works", {
     expect_equivalent(
-        ngms,
+        char_ngrams(c('insurgents','killed', 'in', 'ongoing', 'fighting')),
         c('insurgents_killed', 'killed_in', 'in_ongoing', 'ongoing_fighting')
     )
     
-    expect_equivalent(
-        charNgms,
-        c('insurgents_killed', 'killed_in', 'in_ongoing', 'ongoing_fighting')
-    )
-    
-    expect_warning(quanteda:::tokens_ngrams_character('insurgents killed', 'in', 'ongoing', 'fighting'), 
+    expect_warning(char_ngrams(c('insurgents killed', 'in', 'ongoing', 'fighting')), 
                  "whitespace detected: you may need to run tokens\\(\\) first")
-    
-    expect_warning(quanteda:::tokens_ngrams_character(c('insurgents killed in ongoing fighting'), n = 1, skip = 1), 
-                   "skip argument ignored for n = 1")
 })
 
 test_that("token_skipgrams works", {
@@ -130,10 +114,10 @@ test_that("test there is no competition between threads", {
                        "insurgents_killed_in", "killed_in_ongoing", "in_ongoing_fighting")), 10)
     names(ngrs) <- names(toks)
     
-    # needs to be repeated becasue thread compeition happen at low chance 
-    for(k in 1:1000) {
-        expect_identical(as.list(tokens_ngrams(toks, n = 2:3)), ngrs)
-    }
+    # needs to be repeated because thread competition happen at low chance 
+    expect_true(
+        all(replicate(1000, identical(as.list(tokens_ngrams(toks, n = 2:3)), ngrs)))
+    )
 })
     
 test_that("tokens_ngrams(x, n = ...) works when ntokens(x) < n", {
