@@ -73,31 +73,6 @@ test_that("tokens_recompile: preserves encoding", {
     )
 })
 
-test_that("tokens_recompile: ngrams", {
-    toks <- tokens(c(one = "a b c"))
-
-    expect_equal(
-        as.list(tokens_ngrams(toks, 2:3)),
-        list(one = c("a_b", "b_c", "a_b_c"))
-    )
-
-    expect_equal(
-        attributes(tokens_ngrams(toks, 2:3, concatenator = " "))$meta$object$concatenator,
-        " "
-    )
-
-    expect_equal(
-        attributes(tokens_ngrams(toks, 2:3, concatenator = " "))$meta$object$ngram,
-        2L:3L
-    )
-
-    attr(toks, "types") <- char_ngrams(attr(toks, "types"), 2:3)
-    expect_equal(
-        quanteda:::tokens_recompile(toks, method = "R"),
-        quanteda:::tokens_recompile(toks, method = "C++")
-    )
-})
-
 test_that("tokens_recompile: [ works for tokens", {
     toks <- tokens(c(one = "a b c d",
                      two = "x y z",
@@ -152,7 +127,7 @@ test_that("tokens_recompile: flag use of padding even when it does not reindex t
 
 test_that("non-ascii types are UTF8 encoded", {
     toks <- list(c(1:3))
-    toks <- quanteda:::qatd_cpp_tokens_recompile(toks, c('あ', 'い', 'う', 'え', 'お'))
+    toks <- quanteda:::cpp_tokens_recompile(toks, c('あ', 'い', 'う', 'え', 'お'))
     expect_equal(Encoding(attr(toks, 'types')), rep('UTF-8', 3))
 })
 
@@ -165,19 +140,19 @@ test_that("keep gap and dupli argument works, #1278", {
     )
 
     toks2 <- quanteda:::tokens_recompile(toks, 'C++', gap = TRUE, dup = TRUE)
-    expect_equal(attr(toks2, 'padding'), FALSE)
+    expect_equal(attr(toks2, 'padding'), TRUE)
     expect_equal(attr(toks2, 'types'), c("b", "c"))
     
     toks3 <- quanteda:::tokens_recompile(toks, 'C++', gap = TRUE, dup = FALSE)
-    expect_equal(attr(toks3, 'padding'), FALSE)
+    expect_equal(attr(toks3, 'padding'), TRUE)
     expect_equal(attr(toks3, 'types'), c("b", "c", "c"))
     
     toks4 <- quanteda:::tokens_recompile(toks, 'C++', gap = FALSE, dup = TRUE)
-    expect_equal(attr(toks4, 'padding'), FALSE)
+    expect_equal(attr(toks4, 'padding'), TRUE)
     expect_equal(attr(toks4, 'types'), c("a", "b", "c", "d"))
     
     toks5 <- quanteda:::tokens_recompile(toks, 'C++', gap = FALSE, dup = FALSE)
-    expect_equal(attr(toks5, 'padding'), FALSE)
+    expect_equal(attr(toks5, 'padding'), TRUE)
     expect_equal(attr(toks5, 'types'), c("a", "b", "c", "c", "d"))
     
     expect_equal(quanteda:::tokens_recompile(toks, 'C++', gap = TRUE, dup = TRUE),
